@@ -17,6 +17,8 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#define MAX_SYMBOLLEN 150
+
 #include "defs.h"
 #include "symtab.h"
 #include "gdbtypes.h"
@@ -28,6 +30,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "m3-uid.h"
 #include "frame.h"
 #include "target.h"
+#include "gdb_string.h"
 
 struct symbol * find_m3_ir ();
 
@@ -457,6 +460,7 @@ init_thread_constants ()
 {
   int thread__t__context_size, thread__t__context_offset;
   struct type * thread__t__context_type;
+  int dataOffset = TARGET_PTR_BIT;
 
   if (thread__t) return;
 
@@ -487,13 +491,17 @@ init_thread_constants ()
 		     &thread__t__buf_size, &thread__t__buf_offset, 0);
 
   /* skip past the method pointer */
-  thread__t__id_offset    += TARGET_PTR_BIT;
-  thread__t__state_offset += TARGET_PTR_BIT;
-  thread__t__next_offset  += TARGET_PTR_BIT;
-  thread__t__cond_offset  += TARGET_PTR_BIT;
-  thread__t__mutex_offset += TARGET_PTR_BIT;
-  thread__t__time_offset  += TARGET_PTR_BIT;
-  thread__t__buf_offset   += TARGET_PTR_BIT + thread__t__context_offset;
+#if defined(sparc)
+  /* deal with sparc realignment */
+  dataOffset += TARGET_PTR_BIT;
+#endif
+  thread__t__id_offset    += dataOffset;
+  thread__t__state_offset += dataOffset;
+  thread__t__next_offset  += dataOffset;
+  thread__t__cond_offset  += dataOffset;
+  thread__t__mutex_offset += dataOffset;
+  thread__t__time_offset  += dataOffset;
+  thread__t__buf_offset   += dataOffset + thread__t__context_offset;
 }
 
 /*--------------------------------------------------------- jmpbuf layout ---*/
@@ -691,7 +699,9 @@ threads_command (args, from_tty)
 
   fprintf_filtered (stdout, "-id-   -Thread.T-  -state-\n");
   while (cur.bits) {
-    fprintf_filtered (stdout, "%4d  16_%lx  ", cur.id, cur.ref);
+    fprintf_filtered (stdout, "%4d  ", cur.id);
+    print_longest (stdout, 'x', 1, cur.ref);
+    fprintf_filtered (stdout, "  ");
 
     state = m3_unpack_ord (cur.bits, thread__t__state_offset,
 			   thread__t__state_size, 0);
@@ -1055,9 +1065,9 @@ m3_demangle (mangled)
      char *mangled;
 {
   int i, uid;
-  char demangled [100];
+  char demangled [MAX_SYMBOLLEN];
 
-  char * u;
+  char *u;
 
   if (mangled [0] == 'M' && mangled [2] == '_') {
     switch (mangled[1]) {
@@ -1153,8 +1163,9 @@ m3_demangle (mangled)
   }
 
   /* procedure: *__* */
-  if ((u = strchr (mangled, '_')) && u != mangled && u[1] == '_') {
-    strncpy (demangled, mangled, u - mangled);
+  u = strchr (mangled, '_');
+  if ((u != NULL) && (u != mangled) && (u[1] == '_')) {
+    strncpy (demangled, mangled, (size_t)(u - mangled));
     demangled [u - mangled] = '.';
     strcpy (demangled + (u - mangled) + 1, u + 2);
     return strsave (demangled);
@@ -1288,7 +1299,7 @@ struct type *
 find_m3_type_named (name)
      char *name;
 {
-  char struct_name [100];
+  char struct_name [MAX_SYMBOLLEN];
   struct symbol *s;
 
   sprintf (struct_name, "B$%s", name);
@@ -1307,7 +1318,7 @@ find_m3_exported_interfaces (name)
 	interface; note that if the result is NIL, this means
         that the module exports itself only. */
 {
-  char struct_name [100];
+  char struct_name [MAX_SYMBOLLEN];
   struct symbol *s;
 
   sprintf (struct_name, "H$%s", name);
@@ -1323,7 +1334,7 @@ find_m3_ir (kind, name)
      int kind; 
      char *name;
 {
-  char struct_name [100];
+  char struct_name [MAX_SYMBOLLEN];
   sprintf (struct_name, "%c$%s", kind, name);
   return lookup_symbol (struct_name, 0, VAR_NAMESPACE, 0, 0);
 }
@@ -1333,7 +1344,7 @@ find_m3_type_name (t)
      struct type *t;
 {
   char *uid = TYPE_TAG_NAME (t);
-  char struct_name [100];
+  char struct_name [MAX_SYMBOLLEN];
   struct symbol *sym;
 
   if (TYPE_NAME (t) == 0) {
@@ -1415,11 +1426,7 @@ find_m3_heap_tc_addr (addr)
 		      TARGET_PTR_BIT / TARGET_CHAR_BIT);
 
   /* the typecode is in Modula-3 bits 1..21 */
-#if TARGET_BYTE_ORDER == BIG_ENDIAN
-  typecode = (typecode >> 11) & 0xfffff;
-#else
-  typecode = (typecode >> 1) & 0xfffff;
-#endif
+  typecode = m3_unpack_ord((char *)&typecode, 1, 20, 0);
 
   target_read_memory (rt0u_types_value 
 		      + typecode * TARGET_PTR_BIT / TARGET_CHAR_BIT,
